"use strict";Object.defineProperty(exports,"__esModule",{value:true});exports.runParallel=void 0;
var _process=require("process");

var _execa=_interopRequireDefault(require("execa"));
var _nvexeca=_interopRequireDefault(require("nvexeca"));

var _stdin=require("./stdin.js");
var _colors=require("./colors.js");
var _header=require("./header.js");
var _dry=require("./dry.js");
var _cleanup=require("./cleanup.js");
var _error=require("./error.js");
var _output=require("./output.js");function _interopRequireDefault(obj){return obj&&obj.__esModule?obj:{default:obj}}


const runParallel=async function({
versionRanges,
command,
args,
continueOpt,
opts})
{
if(command===undefined){
return(0,_dry.printVersions)(versionRanges,opts);
}

const stdinOptions=await(0,_stdin.getParallelStdinOptions)();
const colorOptions=(0,_colors.getColorOptions)();
const optsA={
...opts,
dry:true,
...stdinOptions,
...colorOptions,
stdout:"pipe",
stderr:"pipe",
buffer:true,
all:true,
stripFinalNewline:true,
reject:true};


const versions=await startProcesses({
versionRanges,
command,
args,
opts:optsA});


const state={};
await Promise.all([
(0,_cleanup.cleanupProcesses)(versions,continueOpt,state),
runProcesses(versions,continueOpt,state)]);


return state.exitCode;
};exports.runParallel=runParallel;




const startProcesses=async function({versionRanges,command,args,opts}){
const versions=await Promise.all(
versionRanges.map((versionRange)=>
(0,_nvexeca.default)(versionRange,command,args,opts)));


const versionsA=versions.map(startProcess);
return versionsA;
};

const startProcess=function({versionRange,command,args,execaOptions}){
const childProcess=(0,_execa.default)(command,args,execaOptions);
return{childProcess,versionRange};
};



const runProcesses=async function(versions,continueOpt,state){

let index=0;


for await(const{childProcess,versionRange}of versions){
const shouldStop=await runProcess({
childProcess,
versionRange,
continueOpt,
state,
index});






if(shouldStop){
return;
}


index+=1;
}
};

const runProcess=async function({
childProcess,
versionRange,
continueOpt,
state,
index})
{
(0,_header.printVersionHeader)(versionRange);




if(index===0){
childProcess.all.pipe(_process.stdout);
}

try{
const{all}=await childProcess;
(0,_output.writeProcessOutput)(all,_process.stdout,index);
}catch(error){
await(0,_error.handleParallelError)({
error,
versionRange,
continueOpt,
state,
index});

return!continueOpt;
}
};
//# sourceMappingURL=parallel.js.map